/***************************************************************************
 *   Copyright (C) 2009 by Arthur Renato Mello <arthur@mandriva.com        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License as        *
 *   published by the Free Software Foundation; either version 2 of        *
 *   the License or (at your option) version 3 or any later version        *
 *   accepted by the membership of KDE e.V. (or its successor approved     *
 *   by the membership of KDE e.V.), which shall act as a proxy            *
 *   defined in Section 14 of version 3 of the license.                    *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef TIMEOFTHEDAYWALLPAPER_H
#define TIMEOFTHEDAYWALLPAPER_H

#include <QDomElement>
#include <QTimer>

#include <Plasma/DataEngine>
#include <Plasma/Package>
#include <Plasma/Wallpaper>

#include "ui_timeofthedayconfig.h"

typedef struct TimeOfTheDayEvent{
    bool transition;
    QMap<QSize, QString> *fromFiles;
    QMap<QSize, QString> *toFiles;
    int duration;
} TimeOfTheDayEvent;

class TimeOfTheDayWallpaper : public Plasma::Wallpaper
{
    Q_OBJECT

    public:
        TimeOfTheDayWallpaper(QObject* parent, const QVariantList & args);
        ~TimeOfTheDayWallpaper();

        void save(KConfigGroup & config);
        void paint(QPainter * painter, const QRectF & exposedRect);
        QWidget* createConfigurationInterface(QWidget * parent);

    signals:
        void settingsChanged(bool changed = true);

    protected slots:
        void configWidgetDestroyed();
        void colorChanged(const QColor& color);
        void positioningChanged(int index);
        void xmlFileChanged();
        void updateBackground(const QImage &img);
        void updateFadedImage(qreal frame);
        void startWallpaperTransition();
        void nextWallpaper();
        void setWallpaper(const QString& wallpaper);
        void setTransitionWallpaper();

    protected:
        void init(const KConfigGroup & config);
        void calculateGeometry();
        void renderWallpaper(const QString& image = QString());
        void parseXmlFile();
        void setStartTime(QDomElement docElem);
        void addTransition(QDomElement docElem);
        void addStatic(QDomElement docElem);
        QSize chooseAspectRatio(QList<QSize> list);

    private:
        QTime m_currentTime;
        QString m_xmlFileName;
        int m_baseSecs;
        QMap<QTime, TimeOfTheDayEvent> m_events;
        QSize m_size;
        QColor m_color;
        Plasma::Wallpaper::ResizeMethod m_resizeMethod;
        QTimer m_timer;
        QPixmap m_pixmap;
        QPixmap m_oldPixmap;
        QPixmap m_oldFadedPixmap;
        Plasma::Package *m_wallpaperPackage;
        QString m_currentWallpaper;
        QDateTime m_previousModified;
        QString m_img;
        QString m_transitionWallpaper;
        int m_transitionFrames;
        int m_transitionDuration;
        QWidget *m_configWidget;
        Ui::TimeOfTheDayConfig *m_configUi;
};

#endif //PLASMA_PLUGIN_WALLPAPER_TIMEOFTHEDAY_H
